#!/usr/bin/python

"""
    Copyright 2013, 2014 Ýmir Vésteinsson
    http://dayzero.is
    http://ectd.is

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import sys
import os
import xml.etree.ElementTree as ET
import re
import shutil

def main():
  topfolder = sys.argv[1]
  if sys.argv[2]:
    baseline = True
    destfolder = sys.argv[2]
  topfoldercontents = sorted(os.listdir(topfolder))
  sequences = []
  leaves = []
  modifiedFile = ""
  module1 = False
  for directory in topfoldercontents:
    if len(directory) == 4 and directory.isdigit():    
      sequences.append(os.path.join(topfolder,directory))
  outputfile = open(topfolder + os.sep + 'current-dossier.html', 'w')
  outputfile.write('<html><title>Current dossier and file history</title><body>')
  for sequence in sequences:    
    if os.path.isfile(sequence + os.sep + 'm1' + os.sep + 'eu' + os.sep + 'eu-regional.xml'):
      regional = '/m1/eu/eu-regional.xml#'
      m1file = open(os.path.join(sequence + os.sep + 'm1' + os.sep + 'eu' + os.sep + 'eu-regional.xml'))
      module1 = True          
    if os.path.isfile(sequence + os.sep + 'm1' + os.sep + 'us' + os.sep + 'us-regional.xml'):
      regional = '/m1/us/us-regional.xml#'
      m1file = open(os.path.join(sequence + os.sep + 'm1' + os.sep + 'us' + os.sep + 'us-regional.xml'))  
      module1 = True      
    if module1:
      m1root = ET.parse(m1file).getroot()
      m1leafParents = m1root.findall('.//leaf/..')
      for m1Parent in m1leafParents:
        m1leafNodes = m1Parent.findall('.//leaf')
        for m1leaf in m1leafNodes:
          if 'modified-file' in m1leaf.attrib:
            modifiedFile = m1leaf.attrib['modified-file']
          else:
            modifiedFile = ""
          if '{http://www.w3c.org/1999/xlink}href' in m1leaf.attrib:
            xlink = m1leaf.attrib['{http://www.w3c.org/1999/xlink}href']
          else:
            xlink = ""
          m1LeafValues = m1leaf.findtext('title'), m1leaf.attrib['operation'], '../../../' + sequence[-4:] + regional + m1leaf.attrib['ID'], modifiedFile, m1Parent.tag, sequence[-4:] + '/m1/eu/' + xlink
          if m1LeafValues[1] != "delete":
            leaves.append(m1LeafValues)
          modifiedFile = ""
          xlink = ""
          if 'modified-file' in m1leaf.attrib:
            for item in leaves:
              if m1leaf.attrib['modified-file'] == item[2]:         
                outputfile.write('"' + item[0] + '" from sequence ' + re.match('.*/(\d\d\d\d)/.*', item[2]).group(1) + ' was ' + m1leaf.attrib['operation'] + 'd in sequence ' + sequence[-4:] + '<br/>')
                leaves.remove(item)                
      module1 = False
    if sequence + os.sep + 'index.xml':
      sequencefile = open(os.path.join(sequence,'index.xml'))
      root = ET.parse(sequencefile).getroot()   
      leafParents = root.findall('.//leaf/..')
      for parent in leafParents:
        leafNodes = parent.findall('.//leaf')
        for leaf in leafNodes:
          if 'modified-file' in leaf.attrib:
            modifiedFile = leaf.attrib['modified-file']
          else:
            modifiedFile = ""
          if '{http://www.w3c.org/1999/xlink}href' in leaf.attrib:
            xlink = leaf.attrib['{http://www.w3c.org/1999/xlink}href']
          else:
            xlink = ""
          leafValues = leaf.findtext('title'), leaf.attrib['operation'], '../' + sequence[-4:] + '/index.xml#' + leaf.attrib['ID'], modifiedFile, parent.tag, sequence[-4:] + '/' + xlink
          if not leafValues[4] == 'm1-administrative-information-and-prescribing-information' and leafValues[1] != 'delete':
            leaves.append(leafValues)
          if 'modified-file' in leaf.attrib:
            for item in leaves:
              if leaf.attrib['modified-file'] == item[2]:   
                outputfile.write('"' + item[0] + '" from sequence ' + re.match('.*/(\d\d\d\d)/.*', item[2]).group(1) + ' was ' + leaf.attrib['operation'] + 'd in sequence ' + sequence[-4:] + '<br/>')
                leaves.remove(item)
          modifiedFile = ""
          xlink = ""
  outputfile.write('<br/>The current dossier consists of the following ' + str(len(leaves)) + ' documents:<br/><br/><table border="1"><tr><td>sequence</td><td>element</td><td>modified-file attribute to<br/> replace or delete this file</td></tr>')
  for leaf in leaves:    
    sequenceNumber = re.match('.*/(\d\d\d\d)/.*', leaf[2]).group(1)
    outputfile.write('<tr><td>' + sequenceNumber + '</td><td><a href="' + leaf[5] + '">' + leaf[0] + '</a></td><td>' + leaf[2] + '</td></tr>')
    if baseline:   
      endofsequence = leaf[5].find("/") + 1
      startoffilename = leaf[5].rfind("/")
      sourcepath = leaf[5][endofsequence:startoffilename]
      if not os.path.exists(destfolder + os.sep + sourcepath):
        os.makedirs(destfolder + os.sep + sourcepath)
      shutil.copy2(topfolder + leaf[5], destfolder + os.sep + sourcepath)
      print(sourcepath)      
  outputfile.write('</table></body></html>')
  outputfile.close()
  print('Done. See results in ' + topfolder + os.sep + 'current-dossier.html')

if __name__ == '__main__':
  main()
